/*
  Persistence of Vision Ray Tracer Scene Description File

  See README.txt for an introduction, and the comments here
  for the details.

  Requires LightSys 4 for the sky and sunlight:
  http://www.ignorancia.org/en/index.php?page=Lightsys
  
  --
  Jaime Vives Piqueres, 2007-2008.

*/
#version 3.6  // should work with 3.7 betas

// *********************** 
// *** Global settings ***
// *********************** 
global_settings{
 noise_generator 1
 max_trace_level 6
 assumed_gamma 1.5 // seems to work better than 1.0 for outdoors 
}
#default{texture{finish{ambient 0}}}


// *************************
// *** Standard includes ***
// *************************
#include "colors.inc"
#include "textures.inc"
#include "stones.inc"
#include "functions.inc"


// *********************
// *** trace control ***
// *********************
// general feature swiwtches
#declare use_water =1;          // water plane on/off
#declare use_trees =0;          // trees on/off (2=test)
#declare use_cirrus=0;          // use media clouds on/off
#declare use_birds =0;          // fake birds flying on/off
// terrain control (main features, more below) 
#declare heightmap="hf_convert" // TGA for the height field
#declare slopemap="hf_slopemap" // TGA for the slope map
#declare terrain_turbulence=1;  // random turbulence to avoid pixelization on/off
#declare terrain_detail    =1;  // extra deformations to the original hf on/off
#declare terrain_size=<2000,500,2000>; // terrain measures (needed by other includes)
// camera control (position needed by the forest include)
#declare r_cam=seed(151);       // random seed to find camera placements
// random location on the -z axis:
#declare cam_pos=<0,terrain_size.y*2,-terrain_size.z*.49*rand(r_cam)>; 
#declare cam_h_lo=22;           // location height
#declare cam_h_la=26;           // look_at height
#declare cam_zoom=2.4;          // 2.8,3.5,5,...
#declare cam_trans=<0,0,0>;     // fine-tuning


// ******************************
// *** sun, sky and atmosphere **
// ******************************
#declare r_sun=seed(295);   // seed for random sun placement 
// Lightsys includes
#include "CIE.inc"
#include "lightsys.inc"          
// skylight sun position (random seed used if not specified):
#declare Az   =-120; 
#declare Al   =45; 
#declare North=z;
// other skylight params:
#declare Current_Turbidity=2;            // skylight turbidity
#declare haze_alt=terrain_size.y;        // haze altitude
#declare haze_distance=terrain_size.z*4; // haze distance
// create and place the sky dome, sun light and haze:
#include "i_skylight.inc"


// ******************
// *** cloudscape ***
// ******************
#declare r_cirrus    =seed(317); // seed for random cloudscape placement
#declare Cirrus_Intervals    =1; // clouds media intervals
#declare Cirrus_Level      =0.7; // cloudiness for the cirrus shell
#declare Cirrus_Border     =0.2; // softness level for the cirrus borders
#declare Cirrus_Flatness  =.001; // thickness of the cirrus shell
#declare Cirrus_Aperture   =0.9; // zenital aperture in the clouds to avoid shadows
#declare Cirrus_Brightness =1.0; // cirrus brightness multiplier
#declare Cirrus_Height=terrain_size.y+2; // Cirrus shell height
#declare Cirrus_Density=density{ // cirrus shape pattern
  function{
   f_granite(x,y,z)
  } 
  scale .1
  warp{turbulence .33 lambda 3} 
  scale 1000 warp{turbulence 1 octaves 16 lambda 3} scale 1/1000
  rotate 89.9*y
}  
// create and place the clouds:
#include "i_cirrus.inc"


// *************
// *** water ***
// *************
#declare r_water =seed(878);  // water surface seed
#declare Water_Waves     =5;  // # of waves averaged on the surface pattern
#declare Water_Height   =10;  // water level
#declare Water_Scale    =10;  // scale of the waves size
#declare Water_Color=(Gray10+Green*.25+Blue*.25)*.5; // water/media coloring 
// note : using the clouds darkens this color on the water media (?)
#declare Water_Media     =0;  // media on/off
// create and place the water:
#include "i_water.inc"


// ***************
// *** terrain ***
// ***************
#declare r_trr=seed(2680);  // random seed for terrain placement
#declare r_ttx=seed(5671);  // random seed for texture placement
#declare green_level=.70;   // green slope (>.5+green_trans,<1)
#declare green_trans=.05;   // green to rock transition slope
#declare use_imagemaps=0;   // 0=procedural pigments, 1=image_maps (not provided)
#declare terrain_maxg=9;    // isosurface max gradient
// pigment for the detail function:
#declare p_rocks=                      
pigment{
  granite scale 250*<1.5,1,1.5> warp{turbulence .33}
  scale .1 warp{turbulence .33 octaves 9 lambda 2} scale 10
  translate 100000*rand(r_trr)
}
#declare rocks_depth=.1;
/*
// another example for the detail function:
#declare p_rocks=                      
pigment{
 pigment_pattern{
  average
  pigment_map{
   [1 crackle poly_wave .5 scale 40]
   [.5 crackle poly_wave .5 scale 20]
   [.25 crackle poly_wave .5 scale 10]
  }
  scale 3
 }
 scale .1 warp{turbulence .33 octaves 9} scale 10
 translate 100000*rand(r_trr)
}
#declare rocks_depth=.1;
*/
// creates and places the terrain:
#include "hf2iso_terrain.inc"


// *******************
// *** trees forest **
// *******************
#if (use_trees)
#declare r_t=seed(631);     // random placement seed
#declare forest_depth=2000; // depth of the covered area (from the camera to the center)
#declare forest_width=1500; // width of the covered area (centered on the camera)
#declare trees_scale=20;    // scale of the trees
#declare norm_allow=0.5;    // minimum isosurface slope required (0-1)
#declare use_slopemap=0;    // restrict with slope map?
// create and place the trees
#if (use_trees<2)
 // place the trees
 #declare nt=20000;        // number of trees
 #include "hf2iso_forest.inc"
#else
 // use test planes when adjusting forest depth and width
 plane{z,cam_pos.z+forest_depth pigment{Red} finish{ambient 1} no_shadow}
 plane{x,-forest_width*.5 pigment{Green} finish{ambient 1} no_shadow}
 plane{x,forest_width*.5 pigment{Blue} finish{ambient 1} no_shadow}
#end
#end 


// *************
// *** birds ***
// *************
#if (use_birds)
#declare r_pos=seed(255);  // random placement seed
#declare n_flock=150;      // number of birds
// area covered :
#declare flock_area=<terrain_size.x*.5,terrain_size.y,terrain_size.z*.5>; 
// create and place the flock:
#include "i_flock.inc"
object{flock
 translate Water_Height*y
}
#end


// **************
// *** camera ***
// **************
// let it fall over the terrain
#declare Norm=<0,0,0>;
#declare Start=cam_pos;
#declare Inter=trace(terrain,Start,-1*y,Norm);
#if (Inter.y<Water_Height) // avoid underwater cameras
 #declare Inter=<Inter.x,Water_Height+.001,Inter.z>; 
#end
camera{
 location Inter+(cam_h_lo)*y
 up 2.4*y right 3.2*x
 direction cam_zoom*z
 look_at <0,Inter.y+cam_h_la,0>
 translate cam_trans
}

